export default function ($ = jQuery, Drupal, once, dropZones, Sortable) {

  Drupal.LBPlus = {};
  Drupal.LBPlus.SortInstances = [];

  // Drag to move a block.
  Drupal.behaviors.LBPlusSortBlockDrag = {
    attach(context) {
      Array.prototype.forEach.call(
        context.querySelectorAll('.layout-builder.active .js-layout-builder-region'),
        (region) => {
          Drupal.LBPlus.SortInstances.push(Sortable.create(region, {
            draggable: '.layout-builder-block',
            ghostClass: 'ui-state-drop',
            group: 'builder-region',
            onEnd: event => {
              const $item = $(event.item);
              const $from = $(event.from);

              // Check if the region from the event and region for the item match.
              const itemRegion = $item.closest('.js-layout-builder-region');
              if (event.to === itemRegion[0]) {
                // Find the destination delta.
                const deltaTo = $item.closest('[data-layout-delta]').data('layout-delta');
                // If the block didn't leave the original delta use the destination.
                const deltaFrom = $from
                  ? $from.closest('[data-layout-delta]').data('layout-delta')
                  : deltaTo;

                let queryParameters = {
                  delta_from: deltaFrom,
                  delta_to: deltaTo,
                  region_to: itemRegion[0].getAttribute('region'),
                  block_uuid: $item.data('layout-block-uuid'),
                };
                // Include the preceding block if there is one.
                const precesingBlockUuid = $item.prev('[data-layout-block-uuid]').data('layout-block-uuid');
                if (typeof precesingBlockUuid !== 'undefined') {
                  queryParameters.preceding_block_uuid = precesingBlockUuid;
                }
                // Check if the block is coming from a different section storage.
                const nestedStoragePathFrom = $from.closest('[data-nested-storage-path]').data('nested-storage-path');
                if (typeof nestedStoragePathFrom !== 'undefined') {
                  queryParameters.nested_storage_path_from = nestedStoragePathFrom;
                }
                const nestedStoragePathTo = $item.closest('[data-nested-storage-path]').data('nested-storage-path');
                if (typeof nestedStoragePathTo !== 'undefined') {
                  queryParameters.nested_storage_path_to = nestedStoragePathTo;
                }

                const query = new URLSearchParams(queryParameters).toString();

                Drupal.ajax({
                  url: $item.closest('[data-layout-update-url]').data('layout-update-url') + '?' + query,
                }).execute();
              }
            },
          }));
        },
      );
    },
  };

  // Drag to place a new block or blank section.
  Drupal.behaviors.LBPlusAddBlockDrag = {
    attach: function (context, settings) {
      once('addBlockDrag', '.tabbed-content .draggable-block', context).forEach(draggableBlock => {
        Drupal.behaviors.LBPlusAddBlockDrag.initDragging(draggableBlock, 'block');
      });
      once('addSectionDrag', '.tabbed-content .draggable-section', context).forEach(draggableSection => {
        Drupal.behaviors.LBPlusAddBlockDrag.initDragging(draggableSection, 'new_section', false);
      });
    },
    initDragging: (draggable, type, includeRegions = true) => {
      draggable.ondrag = e => {
        dropZones.addSectionDropZones(null, Drupal.t('Add a new section'));
        if (includeRegions) {
          dropZones.addRegionDropZones();
        }
        // Hide the blank page instructions.
        const blankPage = document.getElementById('lb-plus-blank-page');
        if (blankPage) {
          blankPage.style.display = 'none';
        }
      };
      draggable.ondragstart = e => {
        e.dataTransfer.setData('text/json', JSON.stringify({
          id: e.target.id,
          type: type,
        }));
        e.dataTransfer.setData(type, 'true');
      };
      draggable.ondragend = e => {
        dropZones.clearDropZones();
        // Reveal the blank page instructions if the block wasn't placed.
        const blankPage = document.getElementById('lb-plus-blank-page');
        if (blankPage) {
          blankPage.style.display = 'grid';
        }
      };
    },
  };

  // Click the open sidebar link for the user on blank pages.
  Drupal.behaviors.LBPlusClickAddBlock = {
    attach: (context, settings) => {
      once('clickAddBlock', '.blank-page', context).forEach(addBlockLink => {
        addBlockLink.click();
      });
    },
  };

}

// A block was dropped.
document.addEventListener('element-dropped', e => {
  if (e.detail.type === 'block') {
    const draggedBlock = document.getElementById(e.detail.id);
    const dropZone = e.detail.dropEvent.target;
    const type = dropZone.getAttribute('drop-zone-type');
    let destination = {
      type: type,
    };

    if (type === 'region') {
      destination = {
        ...destination,
        section: dropZone.closest('.lb-plus-section').id,
        position: dropZone.getAttribute('position'),
        region: dropZone.closest('.layout__region').getAttribute('region'),
      };
    }
    if (type === 'section') {
      // Pass either the section ID to put this block in front of, or the string
      // "last" to put it at the end.
      destination.section = dropZone.nextElementSibling?.classList.contains('lb-plus-section') ? dropZone.nextElementSibling.id : 'last';
    }

    // Remove the blank page instructions if the block was placed.
    const blankPage = document.getElementById('lb-plus-blank-page');
    if (blankPage) {
      blankPage.remove();
    }

    // Place the block.
    let ajaxConfig = {
      url: '/lb-plus/place-block/' + drupalSettings['LB+'].sectionStorageType + '/' + drupalSettings['LB+'].sectionStorage,
      type: 'POST',
      dataType: 'text',
      progress: {
        type: 'fullscreen',
        message: Drupal.t('Saving block placement...')
      },
      submit: {
        place_block: {
          plugin_id: draggedBlock.id,
          destination: destination,
        },
      },
      error: (xhr, status, error) => {
        // Remove the AJAX progress spinner.
        document.querySelectorAll('.ajax-progress').forEach(progress => {
          progress.remove();
        });
        // Display a message.
        const messages = new Drupal.Message();
        messages.add('<span id="place-block-error">Unable to place the block.</span>', {type: 'error'});
        document.getElementById('place-block-error').scrollIntoView({behavior: 'smooth', block: 'start'});
        console.log(error);
      },
    };

    if (drupalSettings['LB+'].isLayoutBlock) {
      ajaxConfig.url = ajaxConfig.url + '/' + drupalSettings['LB+'].nestedStoragePath;
    }

    let ajax = Drupal.ajax(ajaxConfig);
    ajax.execute();
  }
});
